#!/bin/bash
PATH=$PATH:/usr/bin

if [ -e /etc/udrec_suite/udrec_suite.conf ]
then
	. /etc/udrec_suite/udrec_suite.conf
else
	echo "`date +%H:%M:%S-%d.%m.%Y` -> no udrec_suite.conf found. exiting ..."
	exit 1
fi

write_mail () {
	/usr/lib/sendmail -oi -oem "${MAIL_TO}" <<-EOT
Subject: udrec_suite beendet am `date +%H:%M:%S-%d.%m.%Y`

Die Aufnahme ${REC_FILE_BASE} ist beendet.

Viel Spass,
 sserver

EOT
}

dbox_message () {
	${CURL} "http://${DBOX_IP}/control/message?popup=${REC_FILE_BASE} ist fertig" > /dev/null 2>&1
}

clean_up () {
        rm -f ${LOCK_FILE}

	# delete symlinks
	rm ${MKDVD_TMP_DIR}/${REC_FILE_BASE}_a?.a?

        if [ ${DELETE_DEMUXED} == 1 ]; then
                rm -rf ${DEMUXED_DIR}
        fi
        if [ ${DELETE_STREAM} == 1 ]; then
                rm -f ${MKDVD_TMP_DIR}/${REC_FILE_BASE}.*
        fi
}

usage () {
        echo "Usage: ./custom_udrec.sh /<path>/<stream-basename> !"
        echo "  This script should be called by sserver."
        echo "  e.g.: ./custom_udrec.sh /<path>/RTL_Television_Formel_I__Das_Rennen_20031012_073001"
        echo "  exiting..."
        exit 1
}


START_DATE=`date +%H:%M:%S-%d.%m.%Y`

cd ${MKDVD_TMP_DIR}

# usage
if [ $# -lt 1 ]
then
	usage
fi

if [ $XSERVER_PROBS == "1" ]
then
	${FIX_X1}
	${FIX_X2}
fi

RED='\e[1;31m'
BLUE='\e[1;34m'
CYAN='\e[1;36m'
NC='\e[0m' 	# No Color

# store "stream"-name passed by sserver
REC_FILE_BASE=`echo $1 | sed -e 's/\(.*\)\(\/\)\(.*\)/\3/g'`

DEMUXED_DIR="${MOVIE_DIR}/${REC_FILE_BASE}/DEMUX"
DVD_DIR="${MOVIE_DIR}/${REC_FILE_BASE}/DVD"
ISO_DIR="${MOVIE_DIR}/${REC_FILE_BASE}/ISO"
TS_DIR="${MOVIE_DIR}/${REC_FILE_BASE}"

TARGET="${MOVIE_DIR}/${REC_FILE_BASE}/${REC_FILE_BASE}.mpg"		

# just to be sure ...
sleep 1

# start the show
if [ -e "${MKDVD_TMP_DIR}/${REC_FILE_BASE}.v0" ]
then
	echo "`date +%H:%M:%S-%d.%m.%Y` -> udrec finished."
	echo "`date +%H:%M:%S-%d.%m.%Y` -> processing ${REC_FILE_BASE} ..."

	# checking already running process
	if [ -e ${LOCK_FILE} ]
	then
                echo "`date +%H:%M:%S-%d.%m.%Y` -> lockfile found, sleeping 60sec ..."
		while [ -e ${LOCK_FILE} ]
		do
	       		PID=`cat ${LOCK_FILE}`
	        	echo "`date +%H:%M:%S-%d.%m.%Y` -> waiting for PID:$PID to terminate ..."
			sleep 60
		done
	else
		echo "`date +%H:%M:%S-%d.%m.%Y` -> no lockfile found ..."
		echo $$ > ${LOCK_FILE}
	fi
	
	# searching for aspect-ratio
	echo "`date +%H:%M:%S-%d.%m.%Y` -> creating ${MKDVD_TMP_DIR}/${REC_FILE_BASE}.tmp"

	# create a 1MB mpv-file, jump 100MB into movie.v0
        dd      if=${MKDVD_TMP_DIR}/${REC_FILE_BASE}.v0 \
		of=${MKDVD_TMP_DIR}/${REC_FILE_BASE}_tmp.v0 \
		bs=1024 count=1000 skip=100000 > /dev/null 2>&1
		
	# get aspect-ratio from tmp_file
	echo "`date +%H:%M:%S-%d.%m.%Y` -> searching for aspect-ratio ..."
	FMT=`${JAVA}	-jar ${PX_BIN} \
			-c   ${PX_INI} \
			-o   ${MKDVD_TMP_DIR}/ \
			${MKDVD_TMP_DIR}/${REC_FILE_BASE}_tmp.v0 \
			| grep "video basics" \
			| sed -e 's/\(.*\)(\([0-9]*:[0-9]*\))\(.*\)/\2/g'`

	# tmp_files no longer needed
	rm -f ${MKDVD_TMP_DIR}/${REC_FILE_BASE}_tmp*

        if [ -z ${FMT} ]; then
                echo "`date +%H:%M:%S-%d.%m.%Y` -> no aspect-ratio found ..."
                echo "`date +%H:%M:%S-%d.%m.%Y` -> try different value for \"skip=10000\" in custom_udrec.sh !"
                exit 1
        else
                echo "`date +%H:%M:%S-%d.%m.%Y` -> found \"${FMT}\""
        fi

	# choose ini
	if [ ${FMT} == 16:9 ] && [ ${NO_AD} == 1 ]; then
		PX_INI=${PX_169_INI}	
	else
		PX_INI=${PX_INI}
	fi
	echo "`date +%H:%M:%S-%d.%m.%Y` -> using PX_INI: ${PX_INI}"

	# creating ${MOVIE_DIR}/${REC_FILE_BASE}
	if [ -e ${MOVIE_DIR}/${REC_FILE_BASE} ]; then
		echo "`date +%H:%M:%S-%d.%m.%Y` -> found ${MOVIE_DIR}/${REC_FILE_BASE} ... deleting ..."
		rm -rf ${MOVIE_DIR}/${REC_FILE_BASE}/*
	else
		echo "`date +%H:%M:%S-%d.%m.%Y` -> creating ${MOVIE_DIR}/${REC_FILE_BASE}"
		mkdir -p ${MOVIE_DIR}/${REC_FILE_BASE}
		if [ $? != 0 ]; then
			echo "`date +%H:%M:%S-%d.%m.%Y` -> failed to create ${MOVIE_DIR}/${REC_FILE_BASE}"
			echo "`date +%H:%M:%S-%d.%m.%Y` -> exiting ..."
			clean_up
			exit 1
		fi
	fi

	# demuxing v0 and a0 (a1,a2) via pX
	echo "`date +%H:%M:%S-%d.%m.%Y` -> trying to demux : ${REC_FILE_BASE} (may take a while ...)"

	# rename Audio-Streams. ProjectX overwrites multiple <stream>.mp2 files
	for A_AUDIO in ${MKDVD_TMP_DIR}/${REC_FILE_BASE}.a[012]
	do
		# extension only
		EXT=`echo ${A_AUDIO} | sed -e 's/^.*\.//'`

		# delete _link_ to stream-file, if exists
		[ -e ${MKDVD_TMP_DIR}/${REC_FILE_BASE}_${EXT}.${EXT} ] && rm ${MKDVD_TMP_DIR}/${REC_FILE_BASE}_${EXT}.${EXT}	

		# symlink stream_a0.a0 to stream.a0
		ln -s ${A_AUDIO} ${MKDVD_TMP_DIR}/${REC_FILE_BASE}_${EXT}.${EXT}
	done

	mkdir -p $DEMUXED_DIR
	if [ $? != 0 ]; then
		echo "`date +%H:%M:%S-%d.%m.%Y` -> failed to create ${DEMUXED_DIR}"
		echo "`date +%H:%M:%S-%d.%m.%Y` -> exiting ..."
		clean_up
		exit 1
	fi

	echo -e "$RED"
	${JAVA}	-jar ${PX_BIN} \
		-c   ${PX_INI} \
		-o   $DEMUXED_DIR/ \
		${MKDVD_TMP_DIR}/${REC_FILE_BASE}.v0 \
		${MKDVD_TMP_DIR}/${REC_FILE_BASE}_*.a?
	PX_RC=$?
	echo "`date +%H:%M:%S-%d.%m.%Y` -> ProjectX Returncode: $PX_RC"

	# pX successful ?
	if [ -e $DEMUXED_DIR/${REC_FILE_BASE}.mpv ]; then
		echo -e "$NC"
		echo "`date +%H:%M:%S-%d.%m.%Y` -> ...done."
		# compression of mpv via requant
		if [ ${MPG_COMPRESSION} == "1" ]; then
			echo "`date +%H:%M:%S-%d.%m.%Y` -> compressing ${DEMUXED_DIR}/${REC_FILE_BASE}.mpv (may take a while ...)"
			echo -e "$BLUE"
			cat ${DEMUXED_DIR}/${REC_FILE_BASE}.mpv \
				| ${REQUANT} ${REQUANT_OPTS} \
				> ${DEMUXED_DIR}/${REC_FILE_BASE}_requant.mpv
			mv -f 	${DEMUXED_DIR}/${REC_FILE_BASE}_requant.mpv \
				${DEMUXED_DIR}/${REC_FILE_BASE}.mpv
			echo -e "$NC"
			echo "`date +%H:%M:%S-%d.%m.%Y` -> ... done."
		fi

		echo "`date +%H:%M:%S-%d.%m.%Y` -> mplexing ${TARGET} (may take a while ...)"
		echo -e "$CYAN"

		if [ ${AUDIO_OPTS} == "stereo" ]; then
			${MPLEX}  ${MPLEX_OPTS} \
			 	  -o ${TARGET} \
				  ${DEMUXED_DIR}/${REC_FILE_BASE}.mpv \
				  ${DEMUXED_DIR}/${REC_FILE_BASE}_a0.mp2
                        MPLEX_RC=$?
                        echo "`date +%H:%M:%S-%d.%m.%Y` -> MPLEX Returncode: $MPLEX_RC"
		else
                        ${MPLEX}  ${MPLEX_OPTS} \
                                  -o ${TARGET} \
                                  ${DEMUXED_DIR}/${REC_FILE_BASE}.mpv \
                                  ${DEMUXED_DIR}/${REC_FILE_BASE}_a?.*
                        MPLEX_RC=$?
                        echo "`date +%H:%M:%S-%d.%m.%Y` -> MPLEX Returncode: $MPLEX_RC"
		fi

		echo -e "$NC"	
		if [ -e ${TARGET} ]; then
			echo "`date +%H:%M:%S-%d.%m.%Y` -> ...done."
			echo "`date +%H:%M:%S-%d.%m.%Y` -> file: ${TARGET}"
			echo "`date +%H:%M:%S-%d.%m.%Y` -> has been created."
		else 
			echo "`date +%H:%M:%S-%d.%m.%Y` -> mplex FAILED :("
			echo "`date +%H:%M:%S-%d.%m.%Y` -> exiting ..."
			clean_up
			exit 1
		fi
	else
		echo "`date +%H:%M:%S-%d.%m.%Y` -> \"projectX\" failed to demux :("
		echo "`date +%H:%M:%S-%d.%m.%Y` -> exiting ..."
		clean_up
		exit 1
	fi

        # Creating TS-File
        if [ ${MAKE_TS_FILE} == 1 ]; then
                echo "`date +%H:%M:%S-%d.%m.%Y` -> MAKE_TS_FILE was chosen, tcmplexing mpv and mp2 for better results ..."

                echo -e "$CYAN"
                TCMPLEX_MPG="${TS_DIR}/TS_${REC_FILE_BASE}.mpg"
                ${TCMPLEX} ${TCMPLEX_OPTS} -i ${DEMUXED_DIR}/${REC_FILE_BASE}.mpv \
                                           -p ${DEMUXED_DIR}/${REC_FILE_BASE}_a0.mp2 \
                                           -o ${TCMPLEX_MPG}
                TC_RC=$?
                echo "`date +%H:%M:%S-%d.%m.%Y` -> tcmplex Returncode: $TC_RC"
                echo -e "$NC"

                if [ $? != 0 ]; then
                        echo "`date +%H:%M:%S-%d.%m.%Y` -> failed to create ${TCMPLEX_MPG} !"
                        echo "`date +%H:%M:%S-%d.%m.%Y` -> exiting ..."
                        clean_up
                        exit 1
                fi
                echo "`date +%H:%M:%S-%d.%m.%Y` -> creating TS-File ..."
                echo -e "$RED"
                ${JAVA} -jar ${PX_BIN} \
                        -c   ${PX_TS_INI} \
                        -o   ${TS_DIR} \
                        ${TCMPLEX_MPG}
                PX_RC=$?
                echo "`date +%H:%M:%S-%d.%m.%Y` -> ProjectX Returncode: $PX_RC"
                echo -e "$NC"

                echo "`date +%H:%M:%S-%d.%m.%Y` -> deleting ${TCMPLEX_MPG}. It's no longer needed."
                rm -f ${TCMPLEX_MPG}
        fi

	if [ ${USE_DVDWIZARD} == 1 ]; then
		# creating $DVD_DIR
		echo "`date +%H:%M:%S-%d.%m.%Y` -> creating directory $DVD_DIR ..."
		mkdir -p $DVD_DIR
		if [ $? != 0 ]; then
			echo "`date +%H:%M:%S-%d.%m.%Y` -> failed to create directory $DVD_DIR !"
			echo "`date +%H:%M:%S-%d.%m.%Y` -> exiting ..."
			clean_up
			exit 1
		fi

		echo "`date +%H:%M:%S-%d.%m.%Y` -> building dvd ..."
		echo -e "$BLUE"
		${DVDWIZARD} ${DVDWIZARD_OPTS} -o ${DVD_DIR} \
						  ${TARGET}

		if [ $? == 0 ]; then
			echo -e "$NC"	
			echo "`date +%H:%M:%S-%d.%m.%Y` -> dvdwizard was successful."	
			# the DVD was created, so the mpeg is no longer needed
			# this is useful for saving diskspace	
			if [ ${DELETE_MPG} == 1 ]; then
				echo "`date +%H:%M:%S-%d.%m.%Y` -> deleting ${TARGET}"
				rm -f ${TARGET}
			fi

			if [ ${CREATE_ISO} == 1 ]; then
				# create $ISO_DIR
				echo "`date +%H:%M:%S-%d.%m.%Y` -> creating directory $ISO_DIR ..."
				mkdir -p $ISO_DIR
				if [ $? != 0 ]; then
					echo "`date +%H:%M:%S-%d.%m.%Y` -> failed to create directory $ISO_DIR !"
					echo "`date +%H:%M:%S-%d.%m.%Y` -> exiting ..."
					clean_up
					exit 1
				fi			

				echo "`date +%H:%M:%S-%d.%m.%Y` -> trying to \"mkisofs\" ${DVD_DIR}"
				echo -e "$RED"
				${MKISOFS} ${MKISOFS_OPTS} -o	${ISO_DIR}/${REC_FILE_BASE}.${ISO_TYPE} \
							 	${DVD_DIR}
				if [ $? == 0 ]; then
		                        echo -e "$NC"
					echo "`date +%H:%M:%S-%d.%m.%Y` -> \"mkisofs\" successful."
					# the ISO was created, so the DVD-directory is no longer needed
		                        # this is useful for saving diskspace
					if [ ${DELETE_DVD} == 1 ]; then
						echo "`date +%H:%M:%S-%d.%m.%Y` -> deleting ${DVD_DIR}"
						rm -rf ${DVD_DIR}
					fi
				else
					echo "`date +%H:%M:%S-%d.%m.%Y` -> \"mkisofs\" failed !"
				fi			
			fi
		else
			echo -e "$NC"
			echo "`date +%H:%M:%S-%d.%m.%Y` -> dvdwizard failed, check \"${MOVIE_DIR}/${REC_FILE_BASE}/dvdwizard.log\" for details !"
		fi
	
		# dvdwizard's temp. files
		rm -rf cpics vtsm vmgm.mpg
		mv dvdwizard.log dvdwizard.xml ${MOVIE_DIR}/${REC_FILE_BASE}/
	fi

	END_DATE=`date +%H:%M:%S-%d.%m.%Y`

	if [ ${DBOX_MESSAGE} == 1 ]; then
		echo "`date +%H:%M:%S-%d.%m.%Y` -> sending message to dbox"				
		dbox_message
	fi

	if [ ${WRITE_MAIL} == 1 ]; then
		echo "`date +%H:%M:%S-%d.%m.%Y` -> writing mail"				
		write_mail
	fi

	# kill Xvfb only if one process is running ...
	PROC_COUNT=`ps -ef | grep "custom_udrec.sh" | wc -l`
	if [ `expr $PROC_COUNT` -lt 2 ]; then
		XVFB_PID=`pidof Xvfb`
		[ ! -z $XVFB_PID ] && kill -15 $XVFB_PID
	fi

	echo "`date +%H:%M:%S-%d.%m.%Y` -> FINISHED !"
	echo "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~"

	clean_up	
	
	# finished ...

        exit 0
else
	echo "`date +%H:%M:%S-%d.%m.%Y` -> no ${MKDVD_TMP_DIR}/${REC_FILE_BASE}.v0 found !"
	echo "`date +%H:%M:%S-%d.%m.%Y` -> exiting ..."
	exit 1
fi

